#ifndef deklaracieH
#define deklaracieH

#include <math.h>
#include <inifiles.hpp>

/**
 * Bod v priestore
 */
struct bod
{
  /**
   * Suradnice bodu
   */
  float x,y,z;
};

/**
 * Struct popisujuci detaily nastavenia fitenss funkcie, ako sa ma vyhodnocovat
 */
struct nastavenie_fitness //popisuje, ako sa ma vyhodnocovat automaticka fitness stolicky
{
 // bool automat_ff;

  /**
   * True, ak sa berie do uvahy stabilita celeho obejktu
   */
  bool stabilita;
  /**
   * Hodnota fitness, ktora je udelena stabilnej stolicke
   */
  int stabilita_hodn;

  /**
   * True, ak sa beru do uvahu kriteria pre sedatko
   */
  bool sedatko;
  /**
   * True, ak sa berie do uvahy rovnost sedatka
   */
  bool sedatko_rovnost;
  /**
   * Hodnota fitness, ktoru dostane skoro vodorovne sedatko
   */
  int sed_rov_hodn;
  /**
   * True, ak sa vyhodnocuje fitness za vysku sedatka
   */
  bool sedatko_vyska;
  /**
   * Maximalne hodnota fitness, ktora sa da ziskat za vysku sedatka
   */
  int sed_vys_hodn;
  /**
   * Dolna hranica intervalu, v ktorom sa udeluje maximalna hodnota fitenss za vysku sedatka
   */
  int sed_vys_min_hodn;
  /**
   * Horna hranica intervalu
   */
  int sed_vys_max_hodn;
  /**
   * True, ak sa berie do uvahy stabilita sedatka
   */
  bool sedatko_stabilita;
  /**
   * Maximalna hodnota fitness, ktora sa da ziska za stabilitu sedatka
   */
  int sed_sta_hodn;

  /**
   * True, ak sa maju brat do uvahy nastavenia pre pocet ciar
   */
  bool pocet_ciar;
  /**
   * True, ak sa berie do uvahy ohranicenie zdola na pocet ciar
   */
  bool poc_ciar_min;
  /**
   * Hodnota dolneho limitu na pocet ciar
   */
  int poc_ciar_min_hodn;
  /**
   * Hodnota penalizacie za kazdu ciaru, ktora chyba do dolneho limitu
   */
  int poc_ciar_min_penal;   //penal by mali byt zaporna hodnoty
  /**
   * True, ak sa berie do uvahy ohranicenie zhora na pocet ciar
   */
  bool poc_ciar_max;
  /**
   * Hodnota maxima pre pocet ciar
   */
  int poc_ciar_max_hodn;
  /** 
   * Hodnota postihu za kazdu ciaru, ktore presahuje limit
   */
  int poc_ciar_max_penal;

  /**
   * True, ak sa maju brat do uvahy ohranicenia na pocet ploch
   */
  bool pocet_ploch;
  /**
   * True, ak sa berie do uvahy ohranicenie na minimalny pocet ploch
   */
  bool poc_plo_min;
  /**
   * Hodnota dolneho limitu na pocet ploch
   */
  int poc_plo_min_hodn;
  /**
   * Hodnota penalizacie za kazdu plochu, ktora chyba do splnenia limitu
   */
  int poc_plo_min_penal;
  /**
   * True, ak sa ma brat do uvahy ohranicenie na maximalny pocet ploch
   */
  bool poc_plo_max;
  /**
   * Hodnota limitu na maximalny pocet ploch
   */ 
  int poc_plo_max_hodn;
  /**
   * Hodnota penalizacie za kazdu plochu, ktora prekracuje limit
   */
  int poc_plo_max_penal;

  /**
   * True, ak sa maju brat do uvahy kriteria pre stoly.
   * tieto kriteria sa nevyhodnocuju, ak utvary nie su stoly, t.j. dvojnasobne symetricke utvary s doskou navrchu
   */
  bool stabilita_stola;
  /**
   * True, ak sa berie do uvahy kriterium na rozostup noh stola
   */
  bool stab_nohy;
  /**
   * True, ak sa berie do uvahy kriterium na rozostup bodov, na ktorych spociva doska stola
   */
  bool stab_doska;
  /**
   * Maximalna hodnota fitness, ktora sa da ziskat za splnenie kriteria pre rozosup noh stola
   */
  int stab_noh_hodn;
  /**
   * Maximalna hodnota fitness, ktora sa da ziskat za splnenie kriteria pre rozosup opor dosky stola
   */
  int stab_dos_hodn;

};

/**
 * Struct popisujuci hodnoty fitness za jednotilve kriteria
 */
struct hodnoty_fitness  //popisuje hodnoty fitness za jednotlive kriteria, celkova fitness je sucet tychto ciastkovych fitness
{
  /**
   * Hodnota fitness, ktoru ziskal objekt za svoju stabilitu
   */
  float stabilita;
  /**
   * Hodnota fitness, ktoru ziskal objekt za pritomnost sedatka
   */
  float sed_rovnost;
  /**
   * Hodnota fitness, ktoru ziskal objekt za vysku sedatok
   */
  float sed_vyska;
  /**
   * Hodnota fitness, ktoru ziskal objekt za stabilitu sedatok
   */
  float sed_stabilita;
  /**
   * Hodnota fitness, ktoru ziskal objekt za pocet useciek
   */
  float pocet_useciek;
  /**
   * Hodnota fitness, ktoru ziskal objekt za pocet rovin
   */
  float pocet_rovin;
  /**
   * Hodnota fitness, ktoru ziskal objekt za rozostup noh stola
   */
  float stol_nohy;
  /**
   * Hodnota fitness, ktoru ziskal objekt za rozosup opor dosky
   */
  float stol_doska;
};

typedef int poleint[100];

/**
 * Struct reprezentujuci trojrozmerny vektor
 */
struct vektor
{
  /**
   * Hodnoty zloziek vektora
   */
  float data[3];
};

const p_mut=1.0;             //pravdepodobnost mutacie *100
const p_plochy=30;           //pravdepodobnost toho, ze na ciare bude zavesena plocha *100
const max_velkost_plochy=3;  //max pocet vrcholov okrem 2 hranovych
const priem_dlzka=3;         //priemerna dlzka postranneho zoznamu
const float eps=0.00001;     //epsilon

#endif
