//---------------------------------------------------------------------------

#ifndef genprH
#define genprH

#include "stolicka.h"
#include <stdio.h>
//---------------------------------------------------------------------------

/** 
 * Trieda zabezpecujuca geneticku manipulaciu s populaciou stoliciek.
 */ 
class prostr
{
 public:
  /** 
   * Pole stoliciek - populacia
   */ 
  stolicka *stolicky[1010];    //aspon 2 na steady state GA (pomocne stolicky)

  /** 
   * Pomocne pole stoliciek, pouziva sa pri vytvarani novej populacie
   */ 
  stolicka *nove[1010];

  /** 
   * Pocet stoliciek v poli stolicky
   */ 
  int pocet;

  /** 
   * Pocet stoliciek v poli nove
   */
  int poc_novych;      

  /** 
   * Pozicia aktualnej stolicky v poli stolicky
   */ 
  int akt;       

  /** 
   * Pocitadlo generacii, aktualna generacia
   */ 
  int gen;       

  /** 
   * Nastavenie, ktore urcuje, akym sposobom sa bude vyhodnocvat fitness funkcia stoliciek
   */
  nastavenie_fitness nast; 


  /** 
   * Konstruktor.
   * vyrobi populaciu nahodnych stoliciek
   * @param poc pocet stoliciek, ktore budu v populacii
   */ 
  prostr(int poc);

  /** 
   * Konstruktor.
   * vyrobi populaciu nahodnych stoliciek, ich pocet a dalsie nastavenia su ulozene v ini subore
   * @param *ini ini subor, v ktorom sa nachadzaju ulozene nastavenia pre premenne genetickeho prostredia a nastavenia pre vyhodnocovanie fitness funkcie v stolickach
   */

  prostr(TIniFile *ini);    //nacita nastavenie_fitness z ini suboru

  /** 
   * Destruktor.
   * zrusi vsetky stolicky v populacii
   */ 
  ~prostr();

  /**
   * Prekrizi dve stolicky, cim ich modifikuje
   * @param st1 prva stolicka na krizenie
   * @param st2 druha stolicka na krizenie
   */
  void prekriz(int st1, int st2);

  /**
   * Vyrobi novu generaciu stoliciek.
   * nahradi sucasne stolicky ich potomkami, ktori vznikli krizenim sucasnych stoliciek
   */	
  void generacia();

  /**
   * Najde poziciu stolicky v populacii s minimalnou hodnotou fitness
   * @return pozicia stolicky
   */
  int min_fit();  //pozicia stolicky s minimalnou fitness

  /**
   * Zisti maximalnu hodnotu fitness v populacii
   * @return maximalna hodnota fitness
   */
  int max_fit();  //hodnota maximalnej fitness

  /**
   * Zisti pocet stoliciek v populacii, ktorych fitness je vyssia ako zadana hodnota
   * @param x hodnota fitness
   * @return pocet stoliciek s fitness vyssou ako x
   */ 
  int pocet_lepsich_ako(int x);

  /**
   * Zisti priemernu hodnotu fitness pre vsetky stolicky v populacii
   * @return priemerna hodnota fitness
   */ 
  float priemerna_fitness_celkova();

  /**
   * Zapise do suboru fit_casti.txt informaciu o priemernej hodnote fitness pre kazde kriterium v sucasnej populacii
   */ 
  void priem_fit_po_castiach();
};


#endif
