//---------------------------------------------------------------------------

#ifndef stromH
#define stromH

#include "zoznam.h"
#include "deklaracie.h"
#include "ukladanie.h"
//---------------------------------------------------------------------------

/**
 * Trieda strom sluzi na pracu so stromom, v ktorom je ulozena reprezentacia stolicky.
 * strom je zlozeny zo zoznamov, ma jeden korenovy zoznam, ku ktoremu mozu byt pripojene ine zoznamy v prvkoch zoznamu, ktore obsahuju roviny
 */
class strom
{
  /**
   * Pocet vrcholov rovin, v ktorych je nasledovnikom len pradzny zoznam
   */
  int pocet_moznych_ciar;  //pocet vrcholov rovin, v ktorych je len prazdny zoznam
  
 private:
  /**
   * Pointer na zoznam, z ktoreho bola odobrata cast na krizenie a do ktoreho sa vlepi nova cast
   */
  zoznam *odkrojeny; //zoznam, za ktoreho bola odobrana cast pri krizeni

  /**
   * Pole dlzok utvarov, ak je utvar rovina, tak jeho dlzka je 100+pocet vrcholov roviny
   */
  int dlzky[100];

  /**
   * Pole, v ktorom su ulozene suradnice jednotlivych utvarov - rovin a ciar
   */
  bod utvary[100][30];
  
  /**
   * Pomocne pocitadlo pri vybere zoznamu na krizenie
   */
  int pompoc;    //pomocne pocitadlo pri vybere zoznamu na krizenie

  
  bod pombody[10]; //pole pomocnych bodov pri vytvarani rovnej roviny
  
 public:
  /**
   * Pocet utvarov v strome, pocet ciar+pocet rovin, pocet prvkov v poli utvary
   */
  int pocet_zaznamov;      //pocet utvarov v strome, pocet ciar+pocet rovin

  /**
   * Pocet ciar v strome
   */
  int pocet_ciar;          //pocet ciar v strome

  /**
   * Pocet ploch v strome
   */
  int pocet_ploch;         //pocet rovin v strome

  /**
   * Pointer na korenovy zoznam stromu
   */
  zoznam *koren;

  /** 
   * True, ak su udaje v poliach dlzky a utvary aktualne
   */
  bool aktualne;  //true ak su udaje v poliach dlzky a utvary aktualne
  
 public:
  /**
   * Konstruktor.
   * vytvori prazdny strom, korenovy zoznam je prazdny zoznam
   */
  strom();

  /**
   * Destruktor.
   * ak strom nie je prazdny, vyprazdni vsetky zoznamy, ktore ho tvoria
   */
  ~strom();

 private:
  /**
   * Najde nahodny zoznam v strome, ktory bude pouzity na krizenie.
   * rekurzivna procedura
   * @param *zoz, ktory bude aktualne prechadzany
   * @return jeden zo zoznamov, cez ktore sa preslo
   */
  zoznam* najdi_zoznam_na_krizenie(zoznam *zoz);

  /**
   * Postupne prechadza prvky zoznamu a niektore z nich moze zmenit.
   * rekurzivne sa vola na vsetky zoznamy, ktore su zavesene na plochach tohto zoznamu
   * @param *zoz zoznam, ktory sa aktualne spracovava
   */
  void mutuj_zoznam(zoznam *zoz);

  /**
   * Vyrobi nahodny element.
   * rekurzivna procedura
   * @param vn hlbka vnorenia
   * @return vygenerovany element
   */
  element nahodny_element(int vn);
  
  /**
   * Vyrobi nahodny posun
   * @return nahodny posun
   */
  posun nahodny_posun();
    
  /**
   * Zduplikuje zoznam
   * @param *zoz1 zoznam z ktoreho sa kopiruje jeho obsah
   * @param *zoz2 zoznam, do ktoreho sa da obsah zoz1
   */
  void zduplikuj_zoznam(zoznam *zoz1, zoznam *zoz2);

  /**
   * Vypocitava hodnoty realnych suradnic vsetkych bodov stolicky
   */
  void vyrataj_body();

  /**
   * Spracuje data ulozene v zozname.
   * prida riadok do pola utvary, pricom predchadzajuca usecka je bod1->bod2, 
   * vhladom na tuto usecku sa pocitaju otocenia prvej usecky v zozname, 
   * bod2 je zaciatok novej ciary, ktora je ulozena v tomto zozname. 
   * @param *zoz zoznam, ktory sa spracovava
   * @param bod1 zaciatok predchadzajucej uzecky
   * @param bod2 koniec predchadzajucej usecky a zaciatok ciary ulozene v tomto zozname
   */
  void spracuj_zoznam(zoznam *zoz, bod bod1, bod bod2);

  /**
   * Spracuje plochu ulozenu v aktualnom elemente.
   * prida riadok do pola utvary, pricom prva usecka roviny je bod1->bod2,
   * je to usecka, ku ktorej je rovina pridruzena, t.j. plocha obsahuje aj tuto usecku
   * vzhladom na tuto usecku su pocitane dalsie usecky ohranicujuce plochu
   * @param el element, v ktorom sa nachadza plocha, ktora sa spracovava
   * @param bod1 prvy bod plochy
   * @param bod2 druhy bod plochy
   */
  void spracuj_plochu(element el, bod bod1, bod bod2);

  /** 
   * Rekurzivne rusi zoznam.
   * ak sa tento zoznam vetvi na nejakej rovine, zrusi rekurzivne aj tieto zoznamy
   * @param *zoz zoznam, ktory sa ma zrusit
   */
  void zrus_zoznam(zoznam *zoz);

  /**
   * Pocita vysledny vektor, ktory vznikne otocenim vektoru na otocenie vzhladom na predchadzajuci vektor
   * @param predch predchadzajuci vektor
   * @param vektor_na_otocenie vektor, ktory sa ide otacat
   * @return vysledny vektor
   */
  vektor otocenie(vektor predch, vektor na_otocenie);

  /**
   * Pocita nasledujuci bod, predchadzajuce bodu su bod1, bod2, bod, ktory ratame
   * @param bod1 prvy bod predchadzajucej usecky
   * @param bod2 druhy bod predchadzajucej usecky
   * @param dalsi posun udavajuci velkosti zmeny
   * @return dalsi bod v poradi
   */
  bod nasledujuci_bod(bod bod1, bod bod2, posun dalsi);
    //vyrata nasledujudi bod.. bod1, bod2, nasledujuci bod
    //
 public:

  /**
   * Vyberie nahodne jeden zo vsetkych zoznamov, z ktorych je zlozeny strom
   * @return pointer na vybrany zoznam
   */
  zoznam* zoznam_na_krizenie();

  /**
   * Prida na koniec zoznamu, z ktoreho bola odobrata cast an krizenie cast zoznamu, ktora sa zacina prvkom, na ktory dostavame pointer
   * @param *uk ukazovatel na prvok zoznamu, ktory mame pridat k skratenemu zoznamu
   */
  void pridaj_po_krizeni(prvok *uk);

  /**
   * Postupne prejde cely strom a niektore prvky zmutuje - nahodnym sposobom mierne zmeni ich parametre 
   */
  void mutuj_strom();

  /**
   * Strom sa zduplikuje do stromu specifikovaneho v parametri
   * @param *s ukazovatel na strom, do ktoreho sa ma tento strom zduplikovat
   */
  void zduplikuj_sa(strom *s);

  /**
   * Vyrobi nahodny strom naplneny nahodnymi hodnotami
   */
  void napln_nahodne();

  /**
   * Zistuje, ci je strom prazdny.
   * @return je true, ak je strom prazdny
   */
  bool is_empty();

  /**
   * Spristupnuje pole dlzky na citanie.
   * @result pointer na prvy prvok pola
   */
  int* vrat_dlzky();

  /**
   * Spritupnuje pole utvary na citanie.
   * @result pointer na prvy bod v poli utvary
   */
  bod* vrat_utvary();
};

/**
 * Rata velkost vektora
 * @param v vstupny vektor
 * @return jeho velkost
 */
float velkost_vektora(vektor v);
#endif
