//---------------------------------------------------------------------------

#ifndef ukladanieH
#define ukladanieH

#include "zoznam.h"
//---------------------------------------------------------------------------

/**
 * Struct popisujuci element v poli pompole pri ukladani stoliciek do pola, ekvivalent prvku v zozname
 */
struct strom_pole
{
  /**
   * Element zo zoznamu, popisuje jednu usecku
   */
  element data;
  /**
   * Pole indexov, na tychto poziciach sa nachadzaju nasledovnici tohto elementu
   */
  int nasledovnici[10];
  /**
   * Index, na tejto pozicii sa nachadza nasledovnik tohoto elementu
   */
  int next;
};

/**
 * Trieda na manipulaciu so stolickami. 
 * zapuzdruje metody urcene na ukladanie stoliciek do suborov a na nacitavanie stoliciek zo suborov
 */ 
class uloz
{
 private:
   /**
    * Pocet prvkov v poli pompole
    */
  int pocet;
  
  /**
   * Pole struktur popisujucich kazdy uzol stromu
   */
  strom_pole pompole[100];

 public:
  /** 
   * spristupnuje vlastnosti nacitanej stolicky
   */
  bool symetria, symetria4, doska;

 public:
  /**
   * Konstruktor.
   * vytvori prazdny objekt
   */
  uloz();

 private:
  /**
   * Vlozi do pola pompole novy prvok, vyraba stromovu strukturu v poli
   * @param *p vkladany prvok
   * @return pozicia novovlozeneho prvky v poli pompole
   */
  int daj_do_pola(prvok *p);    //vlozi prvok p do pola, ktore sa potom ulozi do suboru

  /**
   * Vyrobi z nacitaneho pola strom pri nacitavani stolicky zo suboru
   * @return pointer na korenovy zoznam stromu
   */
  zoznam* vyrob_strom();        //vyrobi z nacitaneho pola strom

  /**
   * Spracuje prvok v poli na pozicii x a prida ho do zoznamu pre nacitavani stolicky zo suboru
   * @param x pozicia v poli pompole, na ktorej je prvok, ktory sa spracovava
   * @param *zoz zoznam, do ktoreho sa vlozi prvok ulozeny v poli
   */
  void spracuj_prvok(zoznam *zoz, int x);  //spracovanie 1 prvku pola a jeho pridanie do stromu

 public: 
  /**
   * Ulozi do suboru stolicku spolu s nastaveniami, ako sa maju interpretovat ulozene udaje
   * @param *zac korenovy zoznam stromu, ktory sa ide ukladat
   * @param filename meno suboru, do ktoreho sa stolicka ulozi
   * @param sym hovori o tom, ci ukladana stolicka je zrkadlovo symetricka
   * @param sym4 hovori o tom, ci stolicka je dvojnasobne symetricka
   * @param doska ci ma byt na vrchu utvaru polozena doska
   */
  void uloz_do_suboru(zoznam *zac, AnsiString filename, bool sym, bool sym4, bool doska);

  /**
   * Vynuluje pocet prvkov pola
   */
  void reset();

  /**
   * Nacita zo suboru stolicku a vyrobi prislusnu stromovu strukturu
   * @param filename meno suboru, z ktoreho sa stolicka nacita
   * @return pointer na korenovy zoznam stromu stolicky
   */
  zoznam* nacitaj_zo_suboru(AnsiString filename);
};
#endif
